// ---------------------------------------------------------------------------
// print_html.cpp
//
// Printing HTML files without header/footer lines using Microsoft Internet
// Explorer.
// 
// See Q236777 for details (HOWTO: Change Print Settings for Internet
// Explorer and  WebBrowser Control Programmatically).
//
// 19/Aug/2002: Initial version.
// 07/Jan/2003: Prevent parallel printing threads using a semaphore.
// 08/Jan/2003: ATL compatibility; replaced AfxBeginThread with _beginthread.
//
// Tilman Kuepper, kuepper@xgraphic.de
// ---------------------------------------------------------------------------

#include "stdafx.h"
#include "print_html.h"
#include <process.h>

// ---------------------------------------------------------------------------
// ASSERT macros available...?!
// ---------------------------------------------------------------------------
#if defined(ASSERT)
	#define MY_ASSERT(x) ASSERT(x)
	#pragma message("ASSERT macro available.")
#else
    #if defined(ATLASSERT)
		#define MY_ASSERT(x) ATLASSERT(x)
		#pragma message("ATLASSERT macro available.")
	#else
		#include <assert.h>
		#define MY_ASSERT(x) assert(x)
		#pragma message("ASSERTE, ATLASSERT macros not found.")
	#endif
#endif

// ---------------------------------------------------------------------------
// Name of HTML file...
// ---------------------------------------------------------------------------
CString C_PrintHtml::filename = TEXT("");

// ---------------------------------------------------------------------------
// Prevent parallel printing threads using a semaphore...
// ---------------------------------------------------------------------------
HANDLE C_PrintHtml::printingResource = CreateSemaphore(
	0,                       // LPSECURITY_ATTRIBUTES lpSemaphoreAttributes
	1,                       // LONG lInitialCount
	1,                       // LONG lMaximumCount
	TEXT("PrintHtmlSema"));  // LPCTSTR lpName

// ---------------------------------------------------------------------------
// This thread puts all the pieces together...
// ---------------------------------------------------------------------------
void C_PrintHtml::PrintingThread(void* pParam)
{
	try
	{
		// Get current header/footer settings...
		CString header, footer;
		GetHeaderFooter(header, footer);

		// Remove header/footer strings...
		SetHeaderFooter(TEXT(" "), TEXT(" "));

		// Launch IE to print html page...
		ExecIE();

		// Restore header/footer settings...
		SetHeaderFooter(header, footer);
	}
	catch(CString err)
	{
		MessageBox(
			0,
			err,
			TEXT("Printing failed"),
			MB_ICONWARNING);
	}

	// ...done!
	ReleaseSemaphore(printingResource, 1, 0);
}

// ---------------------------------------------------------------------------
// Read current header/footer settings (get settings from Registry or - if
// not found - return default settings)...
// ---------------------------------------------------------------------------
void C_PrintHtml::GetHeaderFooter(CString &header, CString &footer)
{
	HKEY hKey = 0;

	// Default values for header/footer strings...
	header = TEXT("&w&bPage &p of &P");
	footer = TEXT("&u&b&d");

	// Look for "PageSetup" information at HKEY_CURRENT_USER...
	bool keyFound = true;
	if( ERROR_SUCCESS != RegOpenKeyEx(
		HKEY_CURRENT_USER,
		TEXT("Software\\Microsoft\\Internet Explorer\\PageSetup"),
		0,
		KEY_READ,
		&hKey) )
	{
		// Also try HKEY_LOCAL_MACHINE...
		if( ERROR_SUCCESS != RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			TEXT("Software\\Microsoft\\Internet Explorer\\PageSetup"),
			0,
			KEY_READ,
			&hKey) )
		{
			keyFound = false;
		}
	}

	if(keyFound)
	{	
		const unsigned long BUF_SIZE = 512;

		// Get header string...
		TCHAR headerBuf[BUF_SIZE];
		unsigned long headerSize = BUF_SIZE;

		if( ERROR_SUCCESS == RegQueryValueEx(
			hKey,
			TEXT("header"),
			0,
			0,
			(LPBYTE)headerBuf,
			&headerSize) )
		{
			header = headerBuf;
		}

		// Get footer string...
		TCHAR footerBuf[BUF_SIZE];
		unsigned long footerSize = BUF_SIZE;

		if( ERROR_SUCCESS == RegQueryValueEx(
			hKey,
			TEXT("footer"),
			0,
			0,
			(LPBYTE)footerBuf,
			&footerSize) )
		{
			footer = footerBuf;
		}

		RegCloseKey(hKey);
	}
}

// ---------------------------------------------------------------------------
// Change header/footer settings...
// ---------------------------------------------------------------------------
void C_PrintHtml::SetHeaderFooter(const CString& header, const CString& footer)
{
	// Set header string...
	HKEY hKey = 0;

	if( ERROR_SUCCESS != RegOpenKeyEx(
		HKEY_CURRENT_USER,
		TEXT("Software\\Microsoft\\Internet Explorer\\PageSetup"),
		0,
		KEY_ALL_ACCESS,
		&hKey) )
	{
		throw CString(TEXT("RegOpenKeyEx() failed."));
	}

	if( ERROR_SUCCESS != RegSetValueEx(
		hKey,
		TEXT("header"),
		0,
		REG_SZ,
		(const BYTE*)(const TCHAR*)header,
		header.GetLength()*sizeof(TCHAR)) )
	{
		RegCloseKey(hKey);
		throw CString(TEXT("RegSetValueEx() failed for header."));
	}

	// Set footer string...
	if( ERROR_SUCCESS != RegSetValueEx(
		hKey,
		TEXT("footer"),
		0,
		REG_SZ,
		(const BYTE*)(const TCHAR*)footer,
		footer.GetLength()*sizeof(TCHAR)) )
	{
		RegCloseKey(hKey);
		throw CString(TEXT("RegSetValueEx() failed for footer."));
	}

	RegCloseKey(hKey);
}

// ---------------------------------------------------------------------------
// Launch IE and wait until printing is complete...
// ---------------------------------------------------------------------------
void C_PrintHtml::ExecIE()
{
	SHELLEXECUTEINFO sei;

	ZeroMemory(&sei, sizeof(SHELLEXECUTEINFO));
	sei.cbSize = sizeof(SHELLEXECUTEINFO );

	sei.lpFile = filename;
	sei.fMask = SEE_MASK_NOCLOSEPROCESS;
	sei.lpVerb = TEXT("print");
	sei.nShow = SW_SHOWNORMAL;

	if(ShellExecuteEx(&sei))
	{
		WaitForSingleObject(sei.hProcess, INFINITE);
	}
}

// ---------------------------------------------------------------------------
// Call this "public" function to print html pages...
// ---------------------------------------------------------------------------
void C_PrintHtml::PrintHtml(const CString& filename) 
{
	if(WAIT_OBJECT_0 == WaitForSingleObject(printingResource, 1))
	{
		C_PrintHtml::filename = filename;
		uintptr_t ok = _beginthread(PrintingThread, 0, 0);

		MY_ASSERT(-1 !=  ok);
	}
	else
	{
		MessageBox(
			0,
			TEXT("Please wait until current print job has finished."),
			TEXT("Printer busy"),
			MB_ICONINFORMATION);
	}
}
